@@ -1,21 +1,21 @@
-/* Copyright (C) 1995, 1996, 1998, 1999, 2000, 2001, 2003, 2004, 2006,
- *   2008, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-1996,1998-2001,2003-2004,2006,2008-2013,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -24,19 +24,25 @@
 #endif
 
 #include <alloca.h>
-#include <stdio.h>
 #include <assert.h>
+#include <stdio.h>
 
-#include "libguile/_scm.h"
-#include "libguile/alist.h"
-#include "libguile/hash.h"
-#include "libguile/eval.h"
-#include "libguile/vectors.h"
-#include "libguile/ports.h"
-#include "libguile/bdw-gc.h"
+#include "alist.h"
+#include "bdw-gc.h"
+#include "boolean.h"
+#include "eq.h"
+#include "eval.h"
+#include "gsubr.h"
+#include "hash.h"
+#include "list.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "ports.h"
+#include "procs.h"
+#include "vectors.h"
+#include "weak-table.h"
 
-#include "libguile/validate.h"
-#include "libguile/hashtab.h"
+#include "hashtab.h"
 
 
 
@@ -60,7 +66,7 @@
 static unsigned long hashtable_size[] = {
   31, 61, 113, 223, 443, 883, 1759, 3517, 7027, 14051, 28099, 56197, 112363,
   224717, 449419, 898823, 1797641, 3595271, 7190537, 14381041
-#if SIZEOF_SCM_T_BITS > 4
+#if SIZEOF_UINTPTR_T > 4
   /* vector lengths are stored in the first word of vectors, shifted by
      8 bits for the tc8, so for 32-bit we only get 2^24-1 = 16777215
      elements.  But we allow a few more sizes for 64-bit. */
@@ -1069,11 +1075,5 @@ scm_internal_hash_for_each_handle (scm_t_hash_handle_fn fn, void *closure,
 void
 scm_init_hashtab ()
 {
-#include "libguile/hashtab.x"
+#include "hashtab.x"
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
