@@ -477,6 +477,11 @@ jit_load_args_3(jit_state_t *_jit, jit_operand_t a, jit_operand_t b,
           M(_FGo_, ldxi_f)		\
           M(_FGG_, ldxr_d)		\
           M(_FGo_, ldxi_d)		\
+					\
+          M(_GG__, ldr_atomic)		\
+          M(_GG__, str_atomic)		\
+          M(_GGG_, swap_atomic)		\
+          M(_GGGG, cas_atomic)		\
   					\
           M(_GG__, str_c)		\
           M(_pG__, sti_c)		\
@@ -579,6 +584,10 @@ jit_load_args_3(jit_state_t *_jit, jit_operand_t a, jit_operand_t b,
           M(_G___, jmpr)		\
           M(_p___, jmpi)		\
           M(R____, jmp)			\
+					\
+          M(_p___, jmpi_with_link)	\
+          M(_____, pop_link_register)	\
+          M(_____, push_link_register)	\
   					\
           M(_____, ret)			\
           M(_G___, retr)		\
@@ -595,6 +604,8 @@ jit_load_args_3(jit_state_t *_jit, jit_operand_t a, jit_operand_t b,
           M(_F___, retval_f)		\
           M(_F___, retval_d)		\
   					\
+          M(_____, breakpoint)		\
+  					\
           M(_FF__, negr_f)		\
           M(_FF__, negr_d)		\
           M(_FF__, absr_f)		\
