@@ -1,6 +1,6 @@
 ;;; Continuation-passing style (CPS) intermediate language (IL)
 
-;; Copyright (C) 2013, 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2019 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -314,6 +314,29 @@
       quo
       rem
       mod
+      inexact
+      sqrt
+      abs
+      floor
+      ceiling
+      sin
+      cos
+      tan
+      asin
+      acos
+      atan
+      atan2
+      fsqrt
+      fabs
+      ffloor
+      fceiling
+      fsin
+      fcos
+      ftan
+      fasin
+      facos
+      fatan
+      fatan2
       logand
       logior
       logxor
@@ -394,7 +417,7 @@
                         ($primcall 'load-u64 %tc16-flonum ()))))
          (setk label ($kargs names vars
                        ($continue ktag0 src
-                         ($primcall 'allocate-words/immediate
+                         ($primcall 'allocate-pointerless-words/immediate
                                     `(flonum . ,(match (target-word-size)
                                                   (4 4)
                                                   (8 2)))
@@ -485,7 +508,14 @@
               ;; ((ulsh/immediate (u6? y) x) (ulsh x y))
               (_
                (match (cons name args)
-                 (('allocate-words/immediate)
+                 (((or 'allocate-words/immediate
+                       'allocate-pointerless-words/immediate))
+                  (define op
+                    (match name
+                      ('allocate-words/immediate
+                       'allocate-words)
+                      ('allocate-pointerless-words/immediate
+                       'allocate-pointerless-words)))
                   (match param
                     ((ann . n)
                      (if (u8? n)
@@ -494,7 +524,7 @@
                            (letv n*)
                            (letk kop ($kargs ('n) (n*)
                                        ($continue k src
-                                         ($primcall 'allocate-words ann (n*)))))
+                                         ($primcall op ann (n*)))))
                            (setk label ($kargs names vars
                                          ($continue kop src
                                            ($primcall 'load-u64 n ())))))))))
